/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.apache.wsif.logging.Trc;
import org.apache.wsif.util.WSIFUtils;

/**
 * Container for type mappings that can be used by dynamic providers.
 *
 * @author Alekander Slominski
 * @author Owen Burroughs <owenb@apache.org>
 */
public class WSIFDynamicTypeMap implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
    protected Vector typeMapList = new Vector();
    // Create a Vector of xmlTypes (QNames) for faster lookup of existing mappings
    protected Vector xmlTypes = new Vector();
    protected ArrayList allTypes;

    /**
     * Default constructor
     */
    public WSIFDynamicTypeMap() {
        Trc.entry(this);
        allTypes = new ArrayList();
        Trc.exit();
    }

    /**
     * Constructor
     * @param alist An ArrayList of all custom types from the wsdl
     */
    public WSIFDynamicTypeMap(ArrayList aList) {
        Trc.entry(this, aList);
        allTypes = aList;
        Trc.exit();
    }

    /**
     * Add new mapping between XML and Java type. This method is equivalent to
     * calling mapType(xmlType, javaType, true)
     * @param xmlType The qualified xml name
     * @param javaType The Java class
     * @see #mapType(QName, Class, boolean)
     */
    public void mapType(QName xmlType, Class javaType) {
        Trc.entry(this, xmlType, javaType);
        int i = xmlTypes.indexOf(xmlType);
        if (i == -1) {
        	// No mapping for this type exists so create one.        	
        	typeMapList.add(new WSIFDynamicTypeMapping(xmlType, javaType));
        	xmlTypes.add(xmlType);
        } else {
        	// Mapping for this type already exists so replace it.
        	typeMapList.setElementAt(new WSIFDynamicTypeMapping(xmlType, javaType), i);        	
        }
        Trc.exit();
    }

    /**
     * Add new mapping between XML and Java type.
     * @param xmlType The qualified xml name
     * @param javaType The Java class
     * @param force flag to indicate if mapping should override an existing one
     * for the same xmlType 
     */
    public void mapType(QName xmlType, Class javaType, boolean force) {
        Trc.entry(this, xmlType, javaType);
        int i = xmlTypes.indexOf(xmlType);
        // Only add mapping if no mapping yet exists for this type
        // or if force is set to true
        if (force || (i == -1)) {        	
            if (i == -1) {
            	// No mapping for this type exists so create one.        	
            	typeMapList.add(new WSIFDynamicTypeMapping(xmlType, javaType));
            	xmlTypes.add(xmlType);
            } else {
            	// Mapping for this type already exists so replace it.
            	typeMapList.setElementAt(new WSIFDynamicTypeMapping(xmlType, javaType), i);        	
            }
        }
        Trc.exit();
    }

    /** 
     * Map a package name to a namespace URI 
     * @param namespace The wsdl namespace  
     * @param packageName The name of the Java package 
     */
    public void mapPackage(String namespace, String packageName) {
        Trc.entry(this, namespace, packageName);
        Iterator it = allTypes.iterator();
        while (it.hasNext()) {
            QName qname = (QName) it.next();
            String ns = qname.getNamespaceURI();
            if (ns != null && ns.equals(namespace)) {
                resolveMapping(qname, packageName);
            }
        }
        Trc.exit();
    }

    protected void resolveMapping(QName qn, String pn) {
        Trc.entry(this, qn, pn);
        try {
            String xmlName = qn.getLocalPart();
            if (xmlName != null) {
                String javaName = WSIFUtils.getJavaClassNameFromXMLName(xmlName);
                String classname = pn + "." + javaName;
                Class cl =
                    Class.forName(classname, true, Thread.currentThread().getContextClassLoader());
                mapType(qn, cl);
            }
        } catch (Exception e) {
        	Trc.ignoredException(e);
            //ignore
        }
        Trc.exit();
    }

    /**
     * Set the list of all custom types from the wsdl
     * @param aList The list
     */
    public void setAllTypes(ArrayList aList) {
        Trc.entry(this, aList);
        allTypes = aList;
        Trc.exit();
    }

    /**
     * Return iterator with all mappings.
     * @return The iterator
     */
    public Iterator iterator() {
        Trc.entry(this);
        Trc.exit();
        return typeMapList.iterator();
    }
    /**
     * Produce a copy of the WSIFDynamicTypeMap. This is not a clone;
     * the copy will contain references to the same WSIFDynamicTypeMappings.
     * This method contains synchronized code so that the type map cannot
     * be altered whilst the copy takes place.
     * @return The copy of the WSIFDynamicTypeMap
     */
    public WSIFDynamicTypeMap copy() {
    	// Obtain a lock on the hash map
        synchronized (typeMapList) {
        	WSIFDynamicTypeMap tm = new WSIFDynamicTypeMap();
        	synchronized (allTypes) {
        		tm.setAllTypes((ArrayList) allTypes.clone());
        	}            
            Iterator it = typeMapList.iterator();
            // Copy the mappings
            while (it.hasNext()) {
                WSIFDynamicTypeMapping temp =
                    (WSIFDynamicTypeMapping) it.next();
                tm.mapType(temp.getXmlType(), temp.getJavaType());
            }
            return tm;
        }
    }}