/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.util.jms;

import java.io.Serializable;
import java.util.HashMap;

import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import org.apache.wsif.WSIFException;
import org.apache.wsif.logging.Trc;

/**
 * A WSIFJMSDestination is a pair of queues, one that read from and
 * the other that is written to. This class provides various methods
 * for different flavours of reading and writing messages to those
 * queues. This class hides the JMS interface.
 *
 * @author Mark Whitlock <whitlock@apache.org>
 */
public class WSIFJMSDestination {
    protected WSIFJMSFinder finder;
    protected QueueConnection connection = null;
    protected QueueSession session = null;
    protected Queue readQ = null;
    protected Queue writeQ = null;
    protected QueueSender sender = null;

    protected boolean asyncMode = false;
    protected Queue syncTempQueue = null;

    protected WSIFJMSProperties inProps;
    protected WSIFJMSProperties outProps;
    protected Message lastMessage = null;
    protected long timeout;
    protected String replyToName = null;

    /**
     * Public constructor.
     * @param finder used to find JMS objects.
     */
    public WSIFJMSDestination(WSIFJMSFinder finder) throws WSIFException {
        this(finder, WSIFJMSConstants.WAIT_FOREVER);
        Trc.entry(this, finder);
        Trc.exit();
    }

    /**
     * Public constructor.
     * @param finder used to find JMS objects.
     * @param timeout is the maximum time to wait on a synchronous receive
     */
    public WSIFJMSDestination(WSIFJMSFinder finder, long timeout)
        throws WSIFException {
        this(finder, null, timeout);
        Trc.entry(this, finder, new Long(timeout));
        Trc.exit();
    }

    /**
     * Public constructor.
     * @param finder used to find JMS objects.
     * @param altdestName is an alterative JMS provider destination name
     * @param timeout is the maximum time to wait on a synchronous receive
     */
    public WSIFJMSDestination(
        WSIFJMSFinder finder,
        String altDestName,
        long timeout)
        throws WSIFException {
        Trc.entry(this, finder, altDestName, new Long(timeout));

        inProps = new WSIFJMSProperties(WSIFJMSProperties.IN);
        outProps = new WSIFJMSProperties(WSIFJMSProperties.OUT);
        this.timeout = timeout;
        this.finder = finder;

        try {
            connection = finder.getFactory().createQueueConnection();
            session =
                connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

            Destination initDest = finder.getInitialDestination();
            if (initDest != null && altDestName != null)
                throw new WSIFException("Both jndiDestinationName and jmsproviderDestinationName cannot be specified");
            if (initDest == null && altDestName == null)
                throw new WSIFException("Either jndiDestinationName or jmsproviderDestinationName must be specified");

            if (altDestName != null)
                initDest = session.createQueue(altDestName);

            writeQ = (Queue) initDest;
            readQ = null;

            connection.start();

        } catch (JMSException je) {
            Trc.exception(je);
            throw WSIFJMSConstants.ToWsifException(je);
        }
        if (Trc.ON)
            Trc.exit(deep());
    }

    /**
     * Close all objects.
     */
    public void close() throws WSIFException {
        Trc.entry(this);
        try {
            QueueSender sndr = sender;
            QueueSession sssn = session;
            QueueConnection cnnctn = connection;

            sender = null; // Ensure these are nulled (flagging the close()),
            session = null; // even if a close() throws a JMSException
            connection = null;

            if (sndr != null)
                sndr.close();
            if (sssn != null)
                sssn.close();
            if (cnnctn != null)
                cnnctn.close();
        } catch (JMSException je) {
            Trc.exception(je);
            throw WSIFJMSConstants.ToWsifException(je);
        }
        Trc.exit();
    }

    /**
     * close the destination at finalize.
     */
    public void finalize() throws WSIFException {
        Trc.entry(this);
        close();
        Trc.exit();
    }

    /**
     * Send a message to the write queue
     * @param data is the message
     * @return the id of the message that was sent.
     */
    public String send(String data) throws WSIFException {
        Trc.entry(this, data);
        String s = send(data, null);
        Trc.exit(s);
        return s;
    }

    /**
     * Send a message to the write queue
     * @param data is the message
     * @param id is the correlation id to set on the message
     * @return the id of the message that was sent.
     */
    public String send(String data, String id) throws WSIFException {
        Trc.entry(this, data, id);
        areWeClosed();
        try {
            TextMessage msg = session.createTextMessage();
            msg.setText(data);
            String s = send(msg, id, true);
            Trc.exit(s);
            return s;
        } catch (JMSException je) {
            Trc.exception(je);
            throw WSIFJMSConstants.ToWsifException(je);
        }
    }

    /**
     * Send a message to the write queue
     * @param data is the message
     * @return the id of the message that was sent.
     */
    public String send(Serializable data) throws WSIFException {
        Trc.entry(this, data);
        String s = send(data, null);
        Trc.exit(s);
        return s;
    }

    /**
     * Send a message to the write queue
     * @param data is the message
     * @param id is the correlation id to set on the message
     * @return the id of the message that was sent.
     */
    public String send(Serializable data, String id) throws WSIFException {
        Trc.entry(this, data, id);
        areWeClosed();

        try {
            ObjectMessage msg = session.createObjectMessage();
            msg.setObject(data);
            String s = send(msg, id, true);
            Trc.exit(s);
            return s;
        } catch (JMSException je) {
            Trc.exception(je);
            throw WSIFJMSConstants.ToWsifException(je);
        }
    }

    /**
     * Sends a message to the write queue.
     * @param message
     * @param id Correlation id
     * @param setReplyTo If true JMSReplyTo is always set. If false JMSReplyTo 
     *                    is only set if the ReplyTo was explicitly set as a 
     *                    property.
     */
    public String send(Message msg, String id, boolean setReplyTo)
        throws WSIFException {

        Trc.entry(this, msg, id);
        areWeClosed();

        String msgId = null;
        boolean propsSet = true;

        try {
            if (sender == null)
                sender = session.createSender(writeQ);

            // Process replyTo queues separately since they are not 
            // ordinary JMS properties.
            if (inProps.containsKey(WSIFJMSConstants.REPLY_TO)) {
                String rto = (String) inProps.get(WSIFJMSConstants.REPLY_TO);
                setReplyToQueue(rto);
                inProps.remove(WSIFJMSConstants.REPLY_TO);
                msg.setJMSReplyTo(readQ);
            } else if (setReplyTo) {
                setReplyToQueue();
                msg.setJMSReplyTo(readQ);
            }

            if (id != null)
                msg.setJMSCorrelationID(id);
            inProps.set(sender, msg);

            sender.send(msg);
            msgId = msg.getJMSMessageID();

        } catch (JMSException je) {
            Trc.exception(je);
            throw WSIFJMSConstants.ToWsifException(je);
        } finally {
            // If properties were set, trash the sender so 
            // we get the default props next time.
            if (propsSet)
                sender = null;
            inProps.clear();
        }

        Trc.exit(msgId);
        return msgId;
    }

    /**
     * Blocking receive for the wsif.syncrequest.timeout
     * @return the received message
     */
    public String receive() throws WSIFException {
        Trc.entry(this);
        String s = receiveString(null);
        Trc.exit(s);
        return s;
    }

    /**
     * Blocking receive for the wsif.syncrequest.timeout
     * @return the received message
     */
    public String receiveString(String id) throws WSIFException {
        Trc.entry(this, id);
        String s = receiveString( id, timeout );
        Trc.exit(s);
        return s;
    }

    /**
     * Blocking receive waits for the specified timeout
     * @return the received message
     */
    public String receiveString(String id, long timeout) throws WSIFException {
        Trc.entry(this, id);
        Message msg = receive(id, timeout);
        String s = null;
        try {
            if (msg instanceof TextMessage)
                s = ((TextMessage) msg).getText();
            else
                throw new WSIFException(
                    "Reply message was not a TextMessage:msg="
                        + (msg == null ? "null" : msg.toString()));
        } catch (JMSException e) {
            Trc.exception(e);
            throw WSIFJMSConstants.ToWsifException(e);
        }
        Trc.exit(s);
        return s;
    }

    /**
     * Blocking receive waits for a message for the wsif.syncrequest.timeout
     * @param id is the correlation id that the received message must have
     * @return the received message
     */
    public Message receive(String id) throws WSIFException {
        Trc.entry(this, id);
        Message msg = receive(id, timeout);
        Trc.exit(msg);
        return msg;
    }

    /**
     * Blocking receive waits for a message for the specified timeout
     * @param id is the correlation id that the received message must have
     * @param timeout how long in milliseconds to wait
     * @return the received message
     */
    public Message receive(String id, long timeout) throws WSIFException {
        Trc.entry(this, id);
        areWeClosed();
        QueueReceiver rec = null;
        Message msg = null;

        try {
            if (id != null)
                rec =
                    session.createReceiver(
                        readQ,
                        WSIFJMSConstants.JMS_CORRELATION_ID + "='" + id + "'");
            else
                rec = session.createReceiver(readQ);

            msg = rec.receive(timeout);
            setLastMessage(msg);

            if (msg == null)
                throw new WSIFException(
                    "Receive timed out on JMS queue "
                        + readQ.getQueueName()
                        + ", timeout "
                        + timeout);
        } catch (JMSException e) {
            Trc.exception(e);
            throw WSIFJMSConstants.ToWsifException(e);
        } finally {
            try {
                if (rec != null)
                    rec.close();
            } catch (Exception ignored) {
                Trc.ignoredException(ignored);
            }
        }

        Trc.exit(msg);
        return msg;
    }

    /**
     * Set the replyTo queue to a temporary queue. 
     */
    public void setReplyToQueue() throws WSIFException {
        Trc.entry(this);
        areWeClosed();

        Queue tmp;
        try {
            if (syncTempQueue == null)
                syncTempQueue = session.createTemporaryQueue();
        } catch (JMSException je) {
            Trc.exception(je);
            throw WSIFJMSConstants.ToWsifException(je);
        }

        // So we don't overwrite readQ if there was an error.
        readQ = syncTempQueue;
        replyToName = null;
        Trc.exit();
    }

    /**
     * Set the replyTo queue.
     * @param replyTo queue name.
     */
    public void setReplyToQueue(String replyTo) throws WSIFException {
        Trc.entry(this, replyTo);
        areWeClosed();

        if (replyTo == null || replyTo.length() == 0) {
            setReplyToQueue();
            Trc.exit();
            return;
        }

        // If we're already using this queue, then reuse it.
        if (replyTo.equals(replyToName)) {
            Trc.exit();
            return;
        }

        readQ = finder.findQueue(replyTo);

        replyToName = replyTo;
        Trc.exit();
    }

    /**
     * Sets if this destination is to be used for asynchronous requests.
     * If this destination is to be used for asynchronous requests then a
     * WSIFJMSAsyncListener will be created to listen for the async responses.  
     * 
     * @param b   true if this destination is to be used for asynchronous requests,
     *            otherwise false.
     */
    public void setAsyncMode(boolean b) throws WSIFException {
        Trc.entry(this, b);
        areWeClosed();
        if (asyncMode != b) {
            asyncMode = b;
        }
        Trc.exit();
    }

    /**
     * Sets a JMS property to a value. This property value will be only be used for
     * the next message that is sent, then the property will be reset.
     */
    public void setProperty(String name, Object value) throws WSIFException {
        Trc.entry(this, name, value);
        if (name != null && value != null)
            inProps.put(name, value);
        Trc.exit();
    }

    /**
     * Sets a HashMap of JMS property value pairs. The property values will be only 
     * be used for the next message that is sent, then all the properties will be reset.
     */
    public void setProperties(HashMap propMap) {
        Trc.entry(this, propMap);
        if (propMap != null && !propMap.isEmpty())
            inProps.putAll(propMap);
        Trc.exit();
    }

    /**
     * Gets a JMS property from the previous message that was received.
     */
    public Object getProperty(String name) throws WSIFException {
        Trc.entry(this, name);
        if (lastMessage == null) {
            Trc.exit(null);
            return null;
        }

        if (outProps.isEmpty())
            outProps.getPropertiesFromMessage(lastMessage);

        Object prop = null;
        if (name != null)
            prop = outProps.get(name);

        Trc.exit(prop);
        return prop;
    }

    /**
     * Gets all the JMS properties from the previous message that was received.
     */
    public HashMap getProperties() throws WSIFException {
        Trc.entry(this);
        if (lastMessage == null) {
            Trc.exit(null);
            return null;
        }

        if (outProps.isEmpty())
            outProps.getPropertiesFromMessage(lastMessage);
        if (!outProps.isEmpty()) {
            Trc.exit(outProps);
            return outProps;
        }
        Trc.exit(null);
        return null;
    }

    protected void areWeClosed() throws WSIFException {
        if (session == null)
            throw new WSIFException("Cannot use a closed destination");
    }

    public static Message createMessage(Session session, int msgType)
        throws WSIFException {
        Trc.entry(null, session, new Integer(msgType));
        Message jmsMsg = null;

        try {
            if (msgType
                == org
                    .apache
                    .wsif
                    .wsdl
                    .extensions
                    .jms
                    .JMSConstants
                    .MESSAGE_TYPE_OBJECTMESSAGE)
                jmsMsg = session.createObjectMessage();
            else if (
                msgType
                    == org
                        .apache
                        .wsif
                        .wsdl
                        .extensions
                        .jms
                        .JMSConstants
                        .MESSAGE_TYPE_TEXTMESSAGE)
                jmsMsg = session.createTextMessage();
            else
                throw new WSIFException("Unable to support message type");
        } catch (JMSException je) {
            Trc.exception(je);
            throw WSIFJMSConstants.ToWsifException(je);
        }
        Trc.exit(jmsMsg);
        return jmsMsg;
    }

    public Message createMessage(int msgType) throws WSIFException {
        Trc.entry(this, msgType);
        Message m = createMessage(session, msgType);
        Trc.exit(m);
        return m;
    }

    /**
     * The last message is the most recent message that was received by this 
     * WSIFJMSDestination. The getProperty(s) methods return the properties 
     * that are on the lastMessage. The works fine for sync, but for async
     * user code will have received the message. So the provider must inform
     * the WSIFJMSDestination about the lastMessage explicitly so it can 
     * inquire correctly about any jms properties on it.
     */
    public void setLastMessage(Message msg) {
        Trc.entry(this, msg);
        lastMessage = msg;
        Trc.exit();
    }
    
    public String deep() {
        String buff = "";
        try {
            buff = new String(this.toString() + "\n");
            buff += "finder: " + finder;
            buff += " connection: " + connection;
            buff += " session: " + session;
            buff += " readQ: " + readQ;
            buff += " writeQ: " + writeQ;
            buff += " sender: " + sender;
            buff += " asyncMode: " + asyncMode;
            buff += " syncTempQueue: " + syncTempQueue;
            buff += " inProps: " + inProps;
            buff += " outProps: " + outProps;
            buff += " lastMessage: " + lastMessage;
            buff += " timeout: " + timeout;
            buff += " replyToName: " + replyToName;
        } catch (Exception e) {
            Trc.exceptionInTrace(e);
        }
        return buff;
    }
}