/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.providers.java;

import java.util.Iterator;
import java.util.List;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.wsdl.Port;
import javax.wsdl.Service;
import org.apache.wsif.WSIFException;
import org.apache.wsif.WSIFPort;
import org.apache.wsif.logging.Trc;
import org.apache.wsif.providers.WSIFDynamicTypeMap;
import org.apache.wsif.spi.WSIFProvider;
import org.apache.wsif.wsdl.extensions.java.JavaBinding;

/**
 * Java specific provider of dynamic WSDL invocations.
 * <p>
 * The WSIF Java Provider allows WSIF to invoke Java classes and JavaBeans.
 * This is designed to allow customers in a 'thin-client' environment such as
 * a JVM or Tomcat test runtime to define 'shortcuts' to local Java code. 
 * <p>
 * The Java binding exploits the format binding for type mapping. 
 * The format binding allows WSDL to define the mapping between XML Schema 
 * types and Java types.
 * <p>
 * The Java provider requires the targeted Java classes to be in the 
 * classpath of the client. The Java method is invoked synchronously, 
 * in-process, in-thread, with the current thread and ORB contexts.
 * The Java provider is not transactional.
 * <p>
 * The binding extends WSDL with the following extensibility elements:
 * <p>
 * <pre>
 * &lt;definitions .... &gt; 
 * &lt;!-- Java binding --&gt;
 *    &lt;binding ... &gt;
 *        &lt;java:binding/&gt; 
 *        &lt;format:typeMapping style="uri" encoding="..."/&gt;? 
 *            &lt;format:typeMap name="qname" formatType="nmtoken"/&gt;* 
 *        &lt;/format:typeMapping&gt; 
 *        &lt;operation&gt;* 
 *            &lt;java:operation 
 *                method="nmtoken" 
 *                parameterOrder="nmtoken"? 
 *                methodType="instance|static|constructor"?
 *                returnPart="nmtoken"? /&gt;? 
 *            &lt;input name="nmtoken"? /&gt;? 
 *            &lt;output name="nmtoken"? /&gt;? 
 *            &lt;fault name="nmtoken"? /&gt;? 
 *        &lt;/operation&gt; 
 *    &lt;/binding&gt; 
 *     &lt;service ... &gt; 
 *        &lt;port&gt;* 
 *             &lt;java:address 
 *                 class="nmtoken" 
 *                 archive="uri"? 
 *                 classloader="nmtoken"? /&gt; 
 *        &lt;/port&gt; 
 *    &lt;/service&gt; 
 * &lt;/definitions&gt; 
 * </pre>
 * <p>
 * @author <a href="mailto:gpfau@de.ibm.com">Gerhard Pfau</a>
 * @author <a href="mailto:owenb@apache.org">Owen Burroughs</a> 
 * @author <a href="mailto:antelder@apache.org">Ant Elder</a> 
 * @author <a href="mailto:hughesj@apache.org">Jeremy Hughes</a> 
 * @author <a href="mailto:whitlock@apache.org">Mark Whitlock</a> 
 */
public class WSIFDynamicProvider_Java implements WSIFProvider {
    private static final String[] supportedBindingNamespaceURIs =
        { "http://schemas.xmlsoap.org/wsdl/java/" };

    private static final String[] supportedAddressNamespaceURIs =
        { "http://schemas.xmlsoap.org/wsdl/java/" };

    public WSIFDynamicProvider_Java() {
        Trc.entry(this);
        Trc.exit();
    }

    /**
     * Check if WSDL port has Java binding and if successful try
     * to create Java port instance.
     */
    public WSIFPort createDynamicWSIFPort(
        Definition def,
        Service service,
        Port port,
        WSIFDynamicTypeMap typeMap)
        throws WSIFException {
        Trc.entry(this, def, service, port, typeMap);

        // check that Port binding has Java binding extensibility element
        Binding binding = port.getBinding();
        List exs = binding.getExtensibilityElements();
        for (Iterator i = exs.iterator(); i.hasNext();) {
            Object o = i.next();
            if (o instanceof JavaBinding) {
                // if so try to create Java dynamic port instance
                WSIFPort wp = new WSIFPort_Java(def, port, typeMap);
                Trc.exit(wp);
                return wp;
            }
        }

        // otherwise return null (so other providers can be checked)
        Trc.exit();
        return null;
    }

    /**
     * Returns the WSDL namespace URIs of any bindings this provider supports.
     * @return an array of all binding namespaces supported by this provider
     */
    public String[] getBindingNamespaceURIs() {
        Trc.entry(this);
        Trc.exit(supportedBindingNamespaceURIs);
        return supportedBindingNamespaceURIs;
    }

    /**
     * Returns the WSDL namespace URIs of any port addresses this provider supports.
     * @return an array of all address namespaces supported by this provider
     */
    public String[] getAddressNamespaceURIs() {
        Trc.entry(this);
        Trc.exit(supportedAddressNamespaceURIs);
        return supportedAddressNamespaceURIs;
    }
}